#         Script : UserQuotes v1.00 by David Proper (Dr. Nibble [DrN])
#                  Copyright 2002 Radical Computer Systems
#                             All Rights Reserved
#
#       Testing
#      Platforms : Linux 2.2.16   TCL v8.3 (Patch Level v8.3.1)
#                  Eggdrop v1.6.2
#                  Eggdrop v1.6.6
#            And : SunOS 5.8      TCL v8.3
#                  Eggdrop v1.5.4
#
#    Description : UserQuotes is a simple "tagline" program. The main
#                  script allows OPs to add and delete quotes from users
#                  in the channels. Usally lines taken totally out of
#                  context that sound funny.
#                  And more broadly, there's handlers for read only
#                  tagline files. 
#                  UserQuotes comes with a database of user added quotes
#                  and 60+ files of general taglines.
#                  UserQuotes can also use a random quote to set the
#                  channel topic.
#
# Version History Information is now a seperate file. userquotes.hst
#
#
#   Future Plans : Fix Bugs. :)
#
# Author Contact :     Email - DProper@stx.rr.com
#                  Home Page - http://home.stx.rr.com/dproper
#                        IRC - Nick: DrN  UnderNet/DALnet
#                     UseNet - alt.irc.bots.eggdrop
#
# Chaotix.Net has returned. Web site and mailing list back online
#
#   Radical Computer Systems - http://www.chaotix.net:3000/rcs/
# To subscribe to the RCS mailing list: mail majordomo@chaotix.net and in
#  BODY of message, type  subscribe rcs-list
#
#  Feel free to Email me any suggestions/bug reports/etc.
# 
# You are free to use this TCL/script as long as:
#  1) You don't remove or change author credit
#  2) You don't release it in modified form. (Only the original)
# 
# If you have a "too cool" modification, send it to me and it'll be
# included in the official release. (With your credit)
#
# Commands Added:
#  Where     F CMD          F CMD            F CMD           F CMD
#  -------   - ----------   - ------------   - -----------   - ----------
#  Public:   o !quotes      - !quote
#     MSG:   N/A
#     DCC:   N/A
#
# Public Matching: N/A
#

set quotes(ver) "v1.00.04"
set quotes(format) "(%num%) %quote%"
set quotes(format_info) "(%num%) \"%quote%\" Was Set by %from% on %set%"
set quotes(datafile) "/home/eggdrop/.userquotes"
set quotes(extradir) "/home/eggdrop/userquotes/"
set quotes(lastquotefile) "/home/eggdrop/.lastquote"

set quotes(minlen) 3
set quotes(maxsearch) 25

# Set this to the command charactor to preceed all public commands
set cmdchar_ "*"

proc cmdchar { } {global cmdchar_; return $cmdchar_}


bind pub o|o [cmdchar]quotes pub_quotes
proc pub_quotes {nick uhost hand chan rest} {
 proc_quotes $nick $uhost $hand $chan $rest
}

bind dcc o|o quotes dcc_quotes
proc dcc_quotes {handle idx rest} {
 set chan [string tolower [lindex [console $idx] 0]]
 proc_quotes $idx * $handle $chan $rest
}

proc proc_quotes {nick uhost hand chan rest} {
global quotes userquotes
 set cmd [string toupper [lindex $rest 0]]
 set rest [lrange $rest 1 end]
 switch $cmd { 
 "" {notice_quote $nick "Calling Syntax: [cmdchar]quotes cmd args"
     notice_quote $nick " count           - Display number of quotes"
     notice_quote $nick " add quote       - Add a quote to the database"
     notice_quote $nick " del #           - Delete quote # from the database"
     notice_quote $nick " info #          - Get info on quote #"
     notice_quote $nick " howmany text    - Find how many quotes contain text"
     notice_quote $nick " rand \[database\] - Show a random quote"
     notice_quote $nick " list            - List alternate databases"
     notice_quote $nick " last            - Show last quote added"
     notice_quote $nick " search \[#\] text - Search for quotes containing text."
     notice_quote $nick "                   (Start at \[#\] if given)"
     notice_quote $nick " topic \[database\] - Set the topic using a quote"
    }
 "COUNT" {check_quotes
          putserv "PRIVMSG $chan :$nick: There are $quotes(total) quotes in the database."
         }
 "ADD" {
        if {$rest == ""} {putserv "PRIVMSG $chan :$nick: Uh, what do you want me to add?"
                          putserv "PRIVMSG $chan :\001ACTION mutters the word moron\001"
                          return 0}
        set line $rest
        regsub -all {\\} $line !SL! line
        regsub -all {\<} $line !LT! line
        regsub -all {\>} $line !GT! line
        regsub -all {\"} $line !QT! line
        regsub -all {\(} $line !LP! line
        regsub -all {\)} $line !RP! line
        regsub -all {\{} $line !LB! line
        regsub -all {\}} $line !RB! line
        regsub -all {\[} $line !LF! line
        regsub -all {\]} $line !RF! line
        regsub -all {\;} $line !QL! line
        regsub -all {\:} $line !CO! line

        add_quote $nick $line
        notice_quote $nick "Added Quote: $rest"
        set quotes(lastquote) "$rest"
        set quotes(lastquotewho) "$nick"
        set quotes(lastquotewhen) "[unixtime]"
        save_lastquote
       }
 "DEL" {
        set rest [lindex $rest 0]
        if {$rest == ""} {putserv "PRIVMSG $chan :$nick: Uh, which one do you want me to delete?"
                          putserv "PRIVMSG $chan :\001ACTION mutters the word moron\001"
                          return 0}
        if {![isnum $rest]} {putserv "PRIVMSG $chan :$nick: Which line NUMBER do you want me to delete... Does $rest look like a number to you?"; return 0}
        check_quotes
        if {($rest < 1) || ($rest > $quotes(total))} {putserv "PRIVMSG $chan :$nick: Try using a number from 1 to $quotes(total). Might work better."; return 0}
#        putserv "notice $nick :Removed Quote: [del_quote $rest]"
        notice_quote $nick "[del_quote $rest]"
       }
 "INFO" {
        set rest [lindex $rest 0]
        if {$rest == ""} {putserv "PRIVMSG $chan :$nick: Uh, which one do you want me to  get info on?"
                          putserv "PRIVMSG $chan :\001ACTION mutters the word moron\001"
                          return 0}
        if {![isnum $rest]} {putserv "PRIVMSG $chan :$nick: Which line NUMBER do you want me to get info on... Does $rest look like a number to you?"; return 0}
        check_quotes
        if {($rest < 1) || ($rest > $quotes(total))} {putserv "PRIVMSG $chan :$nick: Try using a number from 1 to $quotes(total). Might work better."; return 0}
#        putserv "PRIVMSG $chan :[info_quote $rest]"
        out_quote $chan [info_quote $rest]
       }
 "HOWMANY" {
            if {$rest == ""} {putserv "PRIVMSG $chan :$nick: Uh, what do you want me to search for?"
                              putserv "PRIVMSG $chan :\001ACTION mutters the word moron\001"
                              return 0}
            putserv "PRIVMSG $chan :$nick: I found [howmany_quotes $chan $nick $rest] matches for $rest"
           }
 "LAST" {
         if {$quotes(lastquote) == ""} {out_quote $chan "There hasn't been any quotes set."; return 0}
         out_quote $chan "Last quote was '$quotes(lastquote)' and was added by $quotes(lastquotewho) on [ctime $quotes(lastquotewhen)]"
        }
 "TOPIC" {
 if {$rest == ""} {quote_topic $chan [rand_quote]
                  } {
                     set desc "[getdesc_userquote $rest] "
                     if {[string range $desc 0 0] == "!"} {set desc ""}
                     quote_topic $chan $desc[rand_userquote $rest]}
}
 "RAND" {#putserv "PRIVMSG $chan :[rand_quote]"
#         out_quote $chan [rand_quote]
 if {$rest == ""} {out_quote $chan "[rand_quote]"
                  } {set desc [getdesc_userquote $rest]
                     if {[string range $desc 0 0] != "!"} {out_quote $chan $desc}
                     out_quote $chan "[rand_userquote $rest]"}
        }
 "LIST" {
         for {set loop 1} {$loop < [expr $quotes(usercnt) + 1]} {incr loop} {
          set line $userquotes($loop)
          set c [lindex $line 0]
          set f [lindex $line 1]
          set d [lrange $line 2 end]
          if {[string range $d 0 0] == "!"} {set d [string range $d 1 [string length $d]]}
          notice_quote $nick "[format "%-15s" $c] $d"
         }
         notice_quote $nick "Listed $quotes(usercnt) entries"
        }
 "SEARCH" {
            if {$rest == ""} {putserv "PRIVMSG $chan :$nick: Uh, what do you want me to search for?"
                              putserv "PRIVMSG $chan :\001ACTION mutters the word moron\001"
                              return 0}
           if {[string length $rest] < $quotes(minlen)} {
            putserv "PRIVMSG $chan :$nick: '$rest' isn't long enough. Must be at least $quotes(minlen) charators."; return 0}
           putserv "PRIVMSG $chan :Searching for quotes matching '$rest' by request of $nick"
           search_quote $nick $chan $rest
          }
              }
}

bind pub - [cmdchar]quote pub_quote
proc pub_quote {nick uhost hand chan rest} {
# putserv "PRIVMSG $chan :[rand_quote]"
 if {$rest == ""} {out_quote $chan "[rand_quote]"
                  } {set desc [getdesc_userquote $rest]
                     if {[string range $desc 0 0] != "!"} {out_quote $chan $desc}
                     out_quote $chan "[rand_userquote $rest]"}
}


proc add_quote {nick text} {
global quotes
  if {![file exists $quotes(datafile)]} {
     set out [open $quotes(datafile) w]
     close $out
                                        }
 set out [open $quotes(datafile) a]
 puts $out "[unixtime] $nick $text"
 close $out
}

proc check_quotes {} {
global quotes
  if {[file exists $quotes(datafile)]} {
    set in [open $quotes(datafile) r]
    set totfound 0

    while {![eof $in]} {
      set line [gets $in]
      if {$line != ""} {set totfound [expr $totfound + 1]}
                       }
    set quotes(total) $totfound
                                } {
                                   set quotes(total) 0
                                  }
                    }

proc rand_quote {} {
global quotes
 check_quotes

 for {set loop 1} {$loop < [expr [rand 100] + 20]} {incr loop} {set pick [rand $quotes(total)]}
 set pick $pick
 set in [open $quotes(datafile) r]
 set line ""
 for {set loop 1} {$loop < [expr $pick + 1]} {incr loop} {set line [gets $in]}
 close $in

 set qtime [lindex $line 0]
 set qfrom [lindex $line 1]
 set quote [lrange $line 2 end]

 set theline $quotes(format)
 regsub -all {%set%} $theline "[ctime $qtime]" theline
 regsub -all {%from%} $theline "$qfrom" theline
 regsub -all {%num%} $theline "$pick" theline
 regsub -all {%quote%} $theline "$quote" theline
 return $theline
                    }

proc del_quote {num} {
global quotes
 check_quotes
 set total 0
 set removed ""

 catch {[exec mv $quotes(datafile) $quotes(datafile).temp]}

 set in [open $quotes(datafile).temp r]
 set out [open $quotes(datafile) w]
 while {![eof $in]} {
  set line [string tolower [gets $in]]
  set total [expr $total + 1]
  if {$total == $num} {set removed $line}
  if {$num != $total} {puts $out $line}
                    }
 close $in
 close $out
 catch {[exec rm $quotes(datafile).temp]}
 set qtime [lindex $removed 0]
 set qfrom [lindex $removed 1]
 set quote [lrange $removed 2 end]
 return $removed
                    }

proc info_quote {pick} {
global quotes
 check_quotes
 set total 0

 set in [open $quotes(datafile) r]
 for {set loop 1} {$loop < [expr $pick + 1]} {incr loop} {set line [gets $in]}
 close $in

 set qtime [lindex $line 0]
 set qfrom [lindex $line 1]
 set quote [lrange $line 2 end]

 set theline $quotes(format_info)
 regsub -all {%set%} $theline "[ctime $qtime]" theline
 regsub -all {%from%} $theline "$qfrom" theline
 regsub -all {%num%} $theline "$pick" theline
 regsub -all {%quote%} $theline "$quote" theline
 return $theline
                    }

proc howmany_quotes {nick chan text} {
global quotes
  if {[file exists $quotes(datafile)]} {
    set in [open $quotes(datafile) r]
    set totfound 0
    set text [string tolower $text]

    while {![eof $in]} {
      set line [string tolower [gets $in]]
      set qtime [lindex $line 0]
      set qfrom [lindex $line 1]
      set quote [lrange $line 2 end]
      if {[string match "*$text*" $quote]} {
                       set totfound [expr $totfound + 1]}
                                          }
                                }
 close $in
 return $totfound
                    }

proc search_quote {nick chan text} {
global quotes
  set firstone 1
  if {[isnum [lindex $text 0]]} {set firstone [lindex $text 0]
                                 set text [lrange $text 1 end]
                                 putserv "PRIVMSG $chan :Listing from found quote number $firstone"
                                }

  if {[file exists $quotes(datafile)]} {
    set in [open $quotes(datafile) r]
    set totfound 0
    set total 0
    set displayed 0
    set text [string tolower $text]

    while {![eof $in]} {
      set line [gets $in]
      set dline [lrange $line 2 end]
      set total [expr $total + 1]
      if {[string match "*$text*" [string tolower $dline]]} {
                       set totfound [expr $totfound + 1]
        if {$totfound >= $firstone} {
         incr displayed 1
         if {$displayed > $quotes(maxsearch)} {
           putserv "PRIVMSG $chan :There are more then $quotes(maxsearch) entries found. Aborting display"
           close $in
           return $totfound
          }
         set qtime [lindex $line 0]
         set qfrom [lindex $line 1]
         set quote [lrange $line 2 end]
         set theline "$totfound $quotes(format)"
         regsub -all {%set%} $theline "[ctime $qtime]" theline
         regsub -all {%from%} $theline "$qfrom" theline
         regsub -all {%num%} $theline "$total" theline
         regsub -all {%quote%} $theline "$quote" theline
#         putserv "PRIVMSG $chan :$theline"
         out_quote $chan $theline
                                   }
                                          }
                      } 
                               } {
                                   set quotes(total) 0
                                  }
 close $in
         out_quote $chan "--EOS"
 return $totfound
                    }

proc out_quote {target txt} {
 regsub -nocase -all {!PD!} $txt \. txt
 regsub -nocase -all {!CO!} $txt \: txt
 regsub -nocase -all {!SQ!} $txt \' txt
 regsub -nocase -all {!QL!} $txt \; txt
 regsub -nocase -all {!QT!} $txt \" txt
 regsub -nocase -all {!LT!} $txt \< txt
 regsub -nocase -all {!GT!} $txt \> txt
 regsub -nocase -all {!LP!} $txt \( txt
 regsub -nocase -all {!RP!} $txt \) txt
 regsub -nocase -all {!LB!} $txt \{ txt
 regsub -nocase -all {!RB!} $txt \} txt
 regsub -nocase -all {!LF!} $txt \[ txt
 regsub -nocase -all {!RF!} $txt \] txt
 regsub -nocase -all {!SL!} $txt \\ txt
 putserv "PRIVMSG $target :$txt"
}

proc quote_topic {chan txt} {
 regsub -nocase -all {!PD!} $txt \. txt
 regsub -nocase -all {!CO!} $txt \: txt
 regsub -nocase -all {!SQ!} $txt \' txt
 regsub -nocase -all {!QL!} $txt \; txt
 regsub -nocase -all {!QT!} $txt \" txt
 regsub -nocase -all {!LT!} $txt \< txt
 regsub -nocase -all {!GT!} $txt \> txt
 regsub -nocase -all {!LP!} $txt \( txt
 regsub -nocase -all {!RP!} $txt \) txt
 regsub -nocase -all {!LB!} $txt \{ txt
 regsub -nocase -all {!RB!} $txt \} txt
 regsub -nocase -all {!LF!} $txt \[ txt
 regsub -nocase -all {!RF!} $txt \] txt
 regsub -nocase -all {!SL!} $txt \\ txt
 putserv "TOPIC $chan :$txt"
}

proc notice_quote {target text} {
if {[isnum $target]} {putidx $target $text} {putserv "NOTICE $target :$text"}
}

# isnumber taken from alltools.tcl
proc isnum {string} {
  if {([string compare $string ""]) && (![regexp \[^0-9\] $string])} then {return 1}
  return 0
}

if {[info exists userquotes]} {unset userquotes}
set quotes(usercnt) 0

proc check_userquotes {filename} {
global quotes
 set totfound 0
  if {[file exists $quotes(extradir)$filename]} {
    set in [open $quotes(extradir)$filename r]
    set totfound 0
    set desc [gets $in]

    while {![eof $in]} {
      set line [gets $in]
      if {$line != ""} {set totfound [expr $totfound + 1]}
                       }
    set quotes(total) $totfound
                                }
return $totfound
                    }

proc getdesc_userquote {cmd} {
global quotes userquotes

 set desc ""
 for {set loop 1} {$loop < [expr $quotes(usercnt) + 1]} {incr loop} {
 set line $userquotes($loop)
 set c [lindex $line 0]
 set f [lindex $line 1]
 set d [lrange $line 2 end]
 if {$c == [string toupper $cmd]} {set desc $d}
 }
 if {$desc == ""} {return "Can't find any quotes file for $cmd"}
 return $desc
}


proc rand_userquote {cmd} {
global quotes userquotes

 set filename ""
 for {set loop 1} {$loop < [expr $quotes(usercnt) + 1]} {incr loop} {
 set line $userquotes($loop)
 set c [lindex $line 0]
 set f [lindex $line 1]
 set d [lrange $line 2 end]
 if {$c == [string toupper $cmd]} {set filename $f}
 }
 if {$filename == ""} {return "Can't find any quotes file for $cmd"}

# return 0
 set total [check_userquotes $filename]
 for {set loop 1} {$loop < [expr [rand 100] + 20]} {incr loop} {set pick [rand $total]}
 set pick $pick
 set in [open $quotes(extradir)$filename r]
 set line ""
 for {set loop 1} {$loop < [expr $pick + 1]} {incr loop} {set line [gets $in]}
 close $in

# set qtime [lindex $line 0]
# set qfrom [lindex $line 1]
# set quote [lrange $line 2 end]
 return  $line

# set theline $quotes(format)
# regsub -all {%set%} $theline "[ctime $qtime]" theline
# regsub -all {%from%} $theline "$qfrom" theline
# regsub -all {%num%} $theline "$pick" theline
# regsub -all {%quote%} $theline "$quote" theline
# return $theline
                    }

set quotes(total) 0

proc load_quotes {} {
global quotes userquotes
 exec rm -f "$quotes(extradir).dir"
 if {[catch {exec /bin/ls -1 ${quotes(extradir)} > ${quotes(extradir)}.dir}]} {
     putlog "ERROR: ls not found on system."
     return 1
    }
 set userlist "" ; set usercnt 0
 set in [open $quotes(extradir).dir]
 while {![eof $in]} {
                gets $in line
                set lline [string tolower $line]
     if {[string range $lline [expr [string length $lline] - 3] [string length $lline]] == ".uq"} {
                incr usercnt 1
                set uqin [open $quotes(extradir)$line]
                gets $uqin desc
                close $uqin
                set cmd [string toupper [string range $lline 0 [expr [string length $lline] - 4]]]
                set userquotes($usercnt) "$cmd $line $desc"
 }
                    }
 close $in
 set quotes(usercnt) $usercnt
 putlog "Loaded $usercnt user quote files"
}
load_quotes

proc save_lastquote {} {
global quotes
# putlog "Saving Last Quote Data"
 set out [open $quotes(lastquotefile) w]
 puts $out $quotes(lastquote)
 puts $out $quotes(lastquotewho)
 puts $out $quotes(lastquotewhen)
 close $out
# timer 60 save_lastquote
}

proc load_lastquote {} {
global quotes
  if {[file exists $quotes(lastquotefile)]} {
                                 set in [open $quotes(lastquotefile) r]
                                 set quotes(lastquote) [gets $in]
                                 set quotes(lastquotewho) [gets $in]
                                 set quotes(lastquotewhen) [gets $in]
                                 close $in
                                } {
                                 set quotes(lastquote) ""
                                 set quotes(lastquotewho) ""
                                 set quotes(lastquotewhen) ""
                                  }
                    }
load_lastquote


putlog "UserQuotes $quotes(ver) by David Proper (DrN) -= LoadeD =-"
return "UserQuotes $quotes(ver) by David Proper (DrN) -= LoadeD =-"

